#ifndef RENDER_H
#define RENDER_H
#include <QObject>
#include <QSize>
#include <QMutex>
#include <QImage>
#include <QFutureWatcher>

enum { ColormapSize = 512 };

struct Chunk {
  Chunk (const int _from, const int _to) : from(_from), to(_to) {};
  const int from, to;
};
struct Wrap {
  Wrap (QImage & _image, const uint * _colormap, 
        const double _scaleFactor, const double _centerX, const double _centerY,
        const int _halfHeight, const int _halfWidth) :
        image(_image), colormap(_colormap),
        scaleFactor(_scaleFactor), centerX(_centerX), centerY(_centerY),
        halfHeight(_halfHeight), halfWidth(_halfWidth){};
  void operator()(const Chunk & chunk);
  private:
    QImage & image;
    const uint * colormap;
    const double scaleFactor, centerX, centerY;
    const int    halfHeight, halfWidth;
};

class Render : public QObject {
    Q_OBJECT
  public:
    Render(QObject * parent = 0);
    ~Render();
    void render (double centerX, double centerY, double scaleFactor,
                 QSize resultSize);
    bool renderImage (QImage & image);
    void setPalette  (int palette);
  public slots:
    void handleFinished ();
  protected:
  signals:
    void renderedImage (const QImage & image, double scaleFactor);
  private:
    QList<Chunk>         chunks;
    QFuture<void>        future;
    QFutureWatcher<void> watcher;
    QImage mandelImage;
    
    double centerX;
    double centerY;
    double scaleFactor;
    QSize  resultSize;
    uint   colormap[ColormapSize];
    
    bool   running;
};

#endif // RENDER_H
